#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2010   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
export LC_TIME="C"

TEMPFILE=$1
TEMPLATE=$2
LOGTYPE=$3
LOCATION=$4
PROPSFILE=$5

ACTUATOR_LOG=../log/$PROPSFILE.log
PROPS=../run/$PROPSFILE
MODTIME=../bin/filemodtime

##################### INTERNAL FUNCTIONS #######################
function printlog {
 echo `date "+%a %D %T"` $1 >> $ACTUATOR_LOG
}

function hpuxEpoch {
  #Good until 2100
  typeset -i year=0
  typeset -i jDay=0
  typeset -i hour=0
  typeset -i min=0
  typeset -i sec=0
  typeset -i totalDays=0
  typeset -i totalHours=0
  typeset -i totalMinutes=0
  typeset -i totalSeconds=0
  date -u +"%Y %j %H %M %S" | read year jDay hour min sec
  (( totalDays = ($year - 1970)*365 + ($year - 1968)/4 + $jDay - 1 ))
  (( totalHours = $totalDays*24 + $hour ))
  (( totalMinutes = $totalHours*60 + $min ))
  (( totalSeconds = $totalMinutes*60 + $sec ))
  echo $totalSeconds
}

function initial_collect {
 printlog "Initial collect ($LOGTYPE), last 2 days included."
 # get current time UNIX-style format (minus 2 days)
 if [ `uname` = "SunOS" ] ; then
  CURRTIME=`nawk 'BEGIN{print srand()}'`
 elif [ `uname` = "HP-UX" ] ; then
  CURRTIME=`hpuxEpoch`
 else
  CURRTIME=`date +%s`
 fi
 TIMESTAMP=`expr $CURRTIME - 172800`
 collect $TIMESTAMP
}

function non_initial_collect {
 printlog "Non-initial collect ($LOGTYPE)"
 TIMESTAMP=$1
 collect $TIMESTAMP
}

function collect {
 TIMESTAMP=$1
 if [ "a$TIMESTAMP" = "a" ] ; then
  printlog "ERROR: Initial timestamp is not defined."
  exit 1
 fi
 printlog "Collect starting from epoch: $TIMESTAMP"
 # Count number of files matching template
 FILESMATCHED=`ls -1 "${LOCATION}" 2>>"${ACTUATOR_LOG}" | grep "${TEMPLATE}" | wc -l`
 if [ $? -ne 0 ] ; then
  printlog "ERROR: Unable to obtain list of files to collect."
  exit 1
 fi
 # Exit if there is no files to collect
 if [ FILESMATCHED -eq 0 ] ; then
  printlog "No files matched template: $TEMPLATE"
  echo "$LOGTYPE=$TIMESTAMP" >> $PROPS.tmp
  exit 0
 fi
 # Exit if only onle file matched
 if [ FILESMATCHED -eq 1 ] ; then
  printlog "Single file found (active). Will not collect it (refer to User Guide)."
  echo "$LOGTYPE=$TIMESTAMP" >> $PROPS.tmp
  exit 0
 fi
 printlog "$FILESMATCHED files matched template. Last modified (active) will not be collected (refer to User Guide)."
 ((FILESMATCHED -= 1))
 N=0
 PROCESSED=0
 LASTTIME=$TIMESTAMP
 # Go through all files matched template in folder specified
 for FILE in `ls -1tr "${LOCATION}" 2>>"${ACTUATOR_LOG}" | grep "${TEMPLATE}"`
  do
  if [ -f "$LOCATION/$FILE" ] ; then
   # File matching template exists, but we need to check modification date
   FILETIME=`$MODTIME "$LOCATION/$FILE" 2>> $ACTUATOR_LOG`
   if [ $? -ne 0 ] ; then
    printlog "ERROR: Failed to get modification time for: $FILE"
    printlog "ERROR: Command failed: $MODTIME \"$LOCATION/$FILE\""
    echo "$LOGTYPE=$LASTTIME" >> $PROPS.tmp
    exit 1
   fi
   ((N += 1))
   ########## We have processed all files except active ##########
   if [ $N -gt $FILESMATCHED ] ; then
    printlog "$PROCESSED files processed. Collect complete."
    echo "$LOGTYPE=$LASTTIME" >> $PROPS.tmp
    exit 0
   fi
   # Check modification time, and copy file to chunk if modification time is greater 
   # then time from props file (we have already skipped most recent/active file)
   if [ $FILETIME -gt $TIMESTAMP ] ; then
    cat $LOCATION/$FILE >> $TEMPFILE 2>> $ACTUATOR_LOG
    if [ $? -ne 0 ] ; then
     printlog "ERROR: Could not copy $FILE to chunk."
     echo "$LOGTYPE=$LASTTIME" >> $PROPS.tmp
     exit 1
    fi
    # File successfully copied to chunk
    printlog "File processed: $FILE"
    ((PROCESSED += 1))
    # Remember file last modified time
    LASTTIME=$FILETIME
   fi
  else
   # File is not accessible
   printlog "ERROR: Could not access file: $FILE"
   printlog "ERROR: Collect for sublog $LOGTYPE aborted. Please check access rights!"
   echo "$LOGTYPE=$LASTTIME" >> $PROPS.tmp
   exit 1   
  fi
 done
}
##################### INTERNAL FUNCTIONS #######################

# Create chunk file
> $TEMPFILE
if [ $? -ne 0 ] ; then 
 printlog "ERROR: Could not create chunk."
 exit 1
fi

# Check incoming parameters
if [ "a$TEMPFILE" = "a" ] ; then 
 printlog "ERROR: Temporary file is not defined for shared script!"
 exit 1
fi
if [ "a$TEMPLATE" = "a" ] ; then 
 printlog "ERROR: Template is not defined for shared script!"
 exit 1
fi
if [ "a$LOGTYPE" = "a" ] ; then 
 printlog "ERROR: Type of the log is not defined for shared script!"
 exit 1
fi
if [ "a$LOCATION" = "a" ] ; then 
 printlog "ERROR: Log files location is not defined for shared script!"
 exit 1
fi
if [ "a$PROPSFILE" = "a" ] ; then 
 printlog "ERROR: PROPS file is not defined for shared script!"
 exit 1
fi

# Check if this is initial collect or not
if [ -f $PROPS ] ; then
 TMP=`cat ${PROPS} | grep "^${LOGTYPE}=" | sed -e "s/^${LOGTYPE}=\(.*\)/\1/"`
 if [ "a$TMP" = "a" ] ; then 
  initial_collect
 else
  non_initial_collect $TMP
 fi
else
 initial_collect
fi

printlog "Unknown error occured (shared script). Exiting."
echo "$LOGTYPE=$LASTTIME" >> $PROPS.tmp
exit 1
