#!/bin/sh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

# Sep 2011 This script collects both the raw audit logs and text-converted logs from the HPUX 11iv3 system
#
# Then it runs audsys to get the current file (or trail), the current file's AFS (automatic file switch).
#
# From the current file's name, it determines the names of all the other audit logs in the same directory, to be collected later
#
# If a search for these logs reveals nothing, then there are no logs to collect (including the current file)
# This indicates that the auditing system is halted, and the last audit file was previously collected (and deleted)
# In this case, we just exit wihtout doing anything.
#
# If not, then we check if the audit system is currently running. If it is, we generate a new filename with the same naming
# convention, and set the auditing system to log to that new file. This avoids any concurrency related issues. In the event
# that the new filename has the same timestamp, and therefore same filename, as the current file, we cannot switch to a new
# file. Therefore we will avoid collecting the file on this run.
#
# The script then calls audisp on the list of files we found. The text output is saved to a temp file, and the raw logs 
# are tar'ed to another temp file. Finally, all the raw logs are deleted.
 ######################################################################################################################

# Die function. Removes tempfiles, optionally prints a message to stderr, and exits.
die ()
{
    [[ -n "$2" ]] && echo "$0: $2" 1>&2
    exit $1
}

####################################################

TEMPFILE1=$1  ## log file(s) after text transformation
TEMPFILE2=$2  ## raw log file(s) 
PROPSFILE=$3  ## currently not used.

if [ -f $TEMPFILE1 ]
then rm $TEMPFILE1
fi

if [ -f $TEMPFILE2 ]
then rm $TEMPFILE2
fi

# Check if we are 'root' user
if [ `id -u` -eq 0 ]
then
    SUDO=""
else
    SUDO="sudo"
fi

# Parse audsys output to find the names of the current and the next
# audit log file.

set `$SUDO audsys | egrep "current (file|trail):"` && CURRENT=$3 && CURRENT_SIZE=$6

# Check if parsing was successful.
[ -n $CURRENT -a -n $CURRENT_SIZE ] || die 3 "Error parsing audsys output."

# Check if auditing is currently on (LS QE110003)
AUDITSTATUS=`$SUDO audsys | grep "currently on"`
if [ -z "$AUDITSTATUS" ] ; then
  AUDITSTATUS=off
else
  AUDITSTATUS=on
fi

# get 'base name' based on current trail
# need to escape curly braces in regex for the annoying sed to work
# the last step will remove the timestamp extension from the filename, if it exists.
TIMESTAMP_PATTERN='\.[0-9]\{8\}_[0-9]\{4\}$'
CURRENT_DIRECTORY=`echo $CURRENT | sed 's@/[^/]*$@@'`
CURRENT_FILENAME=`echo $CURRENT | sed 's@.*/@@'`
BASE_FILENAME=`echo $CURRENT_FILENAME | sed "s@${TIMESTAMP_PATTERN}@@"`

# get list of all filenames. the echo makes it appear as space separated instead of on multiple lines.
TIMESTAMP_PATTERN='\.[0-9]{8}_[0-9]{4}'
FIND=`$SUDO find ${CURRENT_DIRECTORY}`
[ $? -eq 0 ] || die 2 "Error running find on $CURRENT_DIRECTORY"

FILELIST=`$SUDO find ${CURRENT_DIRECTORY} | egrep "$BASE_FILENAME(${TIMESTAMP_PATTERN})?$"`
FILELIST=`echo $FILELIST`

# check for empty file list
# means auditing is off and no new records .. in this case we just do nothing and exit
if [ ! "$FILELIST" ] ; then
   touch $TEMPFILE1
   touch $TEMPFILE2
   touch $PROPSFILE
   echo "Nothing to collect!"
   die 0
fi

if [ $AUDITSTATUS = on ] ; then
  # generate new file name
  NEW_FILE=${CURRENT_DIRECTORY}/${BASE_FILENAME}.`date +%Y%m%d_%H%M`
  
  # in the event that the collect script is run in the same minute as the rollover took place.
  # the script will set the next audit file name to the same as the current one, so no switching takes place
  # then later, there will be an error when trying to delete the file
  # to avoid this, we do not collect the file on this run
  
  if [ $NEW_FILE = $CURRENT ] ; then
    # remove the current file from the list to collect
	FILELIST=`echo $FILELIST | sed "s@$CURRENT@@"`
	# now that filelist has been modified, we need to check this again
	if [ ! "$FILELIST" ] ; then
		touch $TEMPFILE1
		touch $TEMPFILE2
		touch $PROPSFILE
		echo "Nothing to collect!"
		die 0
	fi	
  else
	# switch to new file name
	$SUDO audsys -c $NEW_FILE -s $CURRENT_SIZE
  fi
fi

# run audisp on all filenames
$SUDO audisp -y4 $FILELIST > $TEMPFILE1
[ $? -eq 0 ] || die 2 "Error running audisp command."

# now collect the raw logs
# first we need to create an empty file using the ssh account
# then sudo and add the raw logs to it
# this ensures that the final tar has the ssh user as the owner

touch $TEMPFILE2
$SUDO tar -cf $TEMPFILE2 $FILELIST
[ $? -eq 0 ] || die 2 "Error collecting raw logs (tar)"

touch $PROPSFILE

# delete all the raw logs
$SUDO rm -r $FILELIST

# Clean up and exit successfully.
die 0